local M = {}

local helper = require('scenario/scenariohelper')
local logTag = 'East_Coast_Race_1'

local finalWaypointName = 'way_10'
local playerInstance = 'scenario_player0'
local running = false
local playerWon = false
local aiWon = false

local function reset()
  running = false
  playerWon = false
  aiWon = false
end

local function fail(reason)
  scenario_scenarios.finish({failed = reason})
  reset()
end

local function success(reason)
  scenario_scenarios.finish({msg = reason})
  reset()
end

local function onRaceStart()
  -- log('I', logTag,'onRaceStart called')
  reset()

  local arg = {vehicleName = 'opponent1',
               waypoints = {'way_1', 'way_2', 'way_3', 'way_4', 'way_5', 'way_6', 'way_7', 'way_8',
			               'way_9', finalWaypointName},
              aggression = 0.8, -- aggression here acts as a multiplier to the Ai default aggression i.e. 0.7.
              aggressionMode = 'rubberBand' -- Aggression decreases with distance from opponent
              }

  helper.setAiPath(arg)
  
  local arg = {vehicleName = 'opponent2',
               waypoints = {'way_1', 'way_2', 'way_3', 'way_4', 'way_5', 'way_6', 'way_7', 'way_8',
			               'way_9', finalWaypointName},
              aggression = 0.8, -- aggression here acts as a multiplier to the Ai default aggression i.e. 0.7.
              aggressionMode = 'rubberBand' -- Aggression decreases with distance from opponent
              }

  helper.setAiPath(arg)
  
  local arg = {vehicleName = 'opponent3',
               waypoints = {'way_1', 'way_2', 'way_3', 'way_4', 'way_5', 'way_6', 'way_7', 'way_8',
			               'way_9', finalWaypointName},
              aggression = 0.7, -- aggression here acts as a multiplier to the Ai default aggression i.e. 0.7.
              aggressionMode = 'rubberBand' -- Aggression decreases with distance from opponent
              }

  helper.setAiPath(arg)

  scenario_scenarios.trackVehicleMovementAfterDamage(playerInstance, {waitTimerLimit=2})

	running = true
end

local function onRaceWaypointReached(data)
  --log('I', logTag,'onRaceWaypointReached called ')
  --dump(data)
  if data.waypointName == finalWaypointName then
    if data.vehicleName == playerInstance and not aiWon then
      playerWon = true
    elseif data.vehicleName == aiInstance and not playerWon then
      aiWon = true
    end
  end
end

local function onRaceResult()
  if playerWon then
    success('won')
  else
    fail('lost')
  end
end



M.onRaceStart = onRaceStart
M.onRaceWaypointReached = onRaceWaypointReached
M.onRaceResult = onRaceResult

return M