local M = {}

local helper = require('scenario/scenariohelper')

local running = false
local playerOnCorrectPath = false

local checkForRobberCrash = false
local checkForPoliceDistance = false
local robberCrashTimeout = 0
local robberCrashMaxTimeout = 3

local maxPoliceDamage = 10000
local maxPoliceDistance = 550
local policeDistanceEnable = 50
local robberMinimumSpeed = 1.6 --m/s

local potentialTargets = {'ut_wp_117', 'ut_wp_124', 'ut_wp_125', 'ut_wp_126'}
local targetName = nil

local function reset()
    helper.setAiMode('arms_dealer', 'stopping')
	helper.setAiMode('buyer', 'stopping')
    checkForRobberCrash = false
    checkForPoliceDistance = false
    robberCrashTimeout = 0
    running = false
	
    playerOnCorrectPath = false
end

local function fail(reason)
    scenario_scenarios.finish({failed = reason})
    reset()
end

local function success()
    local scenario = scenario_scenarios.getScenario()
    if not scenario then return end

    local finalTime = scenario.timer
    local minutes = math.floor(finalTime / 60);
    local seconds = finalTime - (minutes * 60);
    local timeStr = ''
    if minutes > 0 then
        timeStr = string.format("%02.0f:%05.2f", minutes, seconds)
    else
        timeStr = string.format("%0.2f", seconds) .. 's'
    end

    local result = {msg = {txt = 'Congratulations! You busted the arms dealer. ', context={timeLimit=timeStr}}}
    scenario_scenarios.finish(result)
    reset()
end

-- called when countdown finished
local function onRaceStart()
    reset()
    running = true
    math.randomseed(os.time())
    targetName = potentialTargets[math.random(#potentialTargets)]

    helper.trackVehicle('scenario_player0', 'policeCar')
    helper.trackVehicle('arms_dealer', 'arms_dealer')
	helper.trackVehicle('buyer', 'buyer')
    helper.queueLuaCommandByName('scenario_player0', 'electrics.set_lightbar_signal(2)')
end

-- data { vehicleName = 'string', waypointName = 'string' }
local function onRaceWaypointReached( data )
    local playerVehicleId = be:getPlayerVehicleID(0)
    if data.vehicleId == playerVehicleId and data.waypointName == 'way_spotted' then
        helper.flashUiMessage('The arms dealer is getting away. Stop him!', 3)
        helper.setAiMode('arms_dealer', 'flee', 'scenario_player0')
		helper.setAiMode('buyer', 'flee', 'scenario_player0')
        helper.setAiTarget('arms_dealer', targetName)
        helper.setAiAggression('arms_dealer', 0.75)
		helper.setAiAggression('buyer', 0.85)
        --helper.queueLuaCommandByName('robber', 'ai.setState({debugMode="speeds"})')
        helper.trackVehicle('arms_dealer', 'arms_dealer')
		
		playerOnCorrectPath = true
		
    end
end


-- called every 250ms. Use "status.setScenarioFailed()" for return failed test
-- data { dTime = number }
local function onRaceTick(raceTickTime)
    if not running then
        return true
    end

    local policeData = map.objects[map.objectNames['policeCar']]
    local robberData = map.objects[map.objectNames['arms_dealer']]

    if not policeData or not robberData then
        return
    end

    local distanceToRobber = math.abs((policeData.pos - robberData.pos):length())
    local robberSpeed = robberData.vel:length()

    if not checkForPoliceDistance and distanceToRobber < policeDistanceEnable then
        checkForPoliceDistance = true
    end

    if not checkForRobberCrash and robberSpeed > robberMinimumSpeed and playerOnCorrectPath == true then
        checkForRobberCrash = true
    end

    if checkForPoliceDistance and distanceToRobber > maxPoliceDistance then
        fail('The dealer escaped. You need to be quick!')
    end

    if policeData.damage > maxPoliceDamage then
        fail('Do not damage your car!')
    end

    if checkForRobberCrash and (robberSpeed < robberMinimumSpeed) then
        robberCrashTimeout = robberCrashTimeout + raceTickTime
        if robberCrashTimeout % 1 == 0 then
            local countdown = robberCrashMaxTimeout - math.modf(robberCrashTimeout)
            if countdown > 0 then
                helper.flashUiMessage({txt = "scenarios.east_coast_usa.bank_robbery.countdown", context = {countdown = tostring(countdown)}}, 0.5, true)
            end
        end
    else
        robberCrashTimeout = 0
    end

    if robberCrashTimeout >= robberCrashMaxTimeout then -- wait x seconds
        success()
    end

    local distance = helper.getDistanceBetweenSceneObjects('arms_dealer', targetName)
    if distance >= 0 and distance <= 550 then
        fail("The dealer got away. You need to be quick!")
    end
end

M.onRaceStart = onRaceStart
M.onRaceWaypointReached = onRaceWaypointReached
M.onRaceTick = onRaceTick

return M
