local M = {}

local helper = require('scenario/scenariohelper')

local running = false
local playerOnCorrectPath = false

local checkForRobberCrash = false
local checkForPoliceDistance = false
local robberCrashTimeout = 0
local robberCrashMaxTimeout = 4

local maxPoliceDamage = 10000
local maxPoliceDistance = 250
local policeDistanceEnable = 60
local robberMinimumSpeed = 1.6 --m/s

local potentialTargets = {'junction1_wp24'}
local targetName = nil

local function reset()
    helper.setAiMode('burglars_getaway', 'stopping')
    checkForRobberCrash = false
    checkForPoliceDistance = false
    robberCrashTimeout = 0
    running = false
	
    playerOnCorrectPath = false
end

local function fail(reason)
    scenario_scenarios.finish({failed = reason})
    reset()
end

local function success()
    local scenario = scenario_scenarios.getScenario()
    if not scenario then return end

    local finalTime = scenario.timer
    local minutes = math.floor(finalTime / 60);
    local seconds = finalTime - (minutes * 60);
    local timeStr = ''
    if minutes > 0 then
        timeStr = string.format("%02.0f:%05.2f", minutes, seconds)
    else
        timeStr = string.format("%0.2f", seconds) .. 's'
    end

    local result = {msg = {txt = 'Not a good time for burglars. Well done, Officer.', context={timeLimit=timeStr}}}
    scenario_scenarios.finish(result)
    reset()
end

-- called when countdown finished
local function onRaceStart()
    reset()
    running = true
    math.randomseed(os.time())
    targetName = potentialTargets[math.random(#potentialTargets)]

    helper.trackVehicle('scenario_player0', 'policeCar')
    helper.trackVehicle('burglars_getaway', 'burglars_getaway')
    helper.queueLuaCommandByName('scenario_player0', 'electrics.set_lightbar_signal(1)')
	helper.queueLuaCommandByName('burglars_getaway', 'electrics.toggle_lights(2)')
end

-- data { vehicleName = 'string', waypointName = 'string' }
local function onRaceWaypointReached( data )
    local playerVehicleId = be:getPlayerVehicleID(0)
    if data.vehicleId == playerVehicleId and data.waypointName == 'way_14' then
        helper.flashUiMessage('Seems like he spotted you. Catch him before he vanishes.', 3)
        helper.setAiMode('burglars_getaway', 'flee', 'scenario_player0')
        helper.setAiTarget('burglars_getaway', targetName)
        helper.setAiAggression('burglars_getaway', 0.75)
        helper.queueLuaCommandByName('burglars_getaway', 'ai.setState({debugMode="speeds"})')
        helper.trackVehicle('burglars_getaway', 'burglars_getaway')
		
		playerOnCorrectPath = true
		
    end
end


-- called every 250ms. Use "status.setScenarioFailed()" for return failed test
-- data { dTime = number }
local function onRaceTick(raceTickTime)
    if not running then
        return true
    end

    local policeData = map.objects[map.objectNames['policeCar']]
    local robberData = map.objects[map.objectNames['burglars_getaway']]

    if not policeData or not robberData then
        return
    end

    local distanceToRobber = math.abs((policeData.pos - robberData.pos):length())
    local robberSpeed = robberData.vel:length()

    if not checkForPoliceDistance and distanceToRobber < policeDistanceEnable then
        checkForPoliceDistance = true
    end

    if not checkForRobberCrash and robberSpeed > robberMinimumSpeed and playerOnCorrectPath == true then
        checkForRobberCrash = true
    end

    if checkForPoliceDistance and distanceToRobber > maxPoliceDistance then
        fail('You couldnt keep up with his car. Total disgrace to BCPD.')
    end

    if policeData.damage > maxPoliceDamage then
        fail('Excessive damage to your car.')
    end

    if checkForRobberCrash and (robberSpeed < robberMinimumSpeed) then
        robberCrashTimeout = robberCrashTimeout + raceTickTime
        if robberCrashTimeout % 1 == 0 then
            local countdown = robberCrashMaxTimeout - math.modf(robberCrashTimeout)
            if countdown > 0 then
                helper.flashUiMessage({txt = "scenarios.east_coast_usa.bank_robbery.countdown", context = {countdown = tostring(countdown)}}, 0.5, true)
            end
        end
    else
        robberCrashTimeout = 0
    end

    if robberCrashTimeout >= robberCrashMaxTimeout then -- wait x seconds
        success()
    end

    local distance = helper.getDistanceBetweenSceneObjects('burglars_getaway')
    if distance >= 0 and distance <= 250 then
        fail("You couldnt keep up with his car. Total disgrace to BCPD.")
    end
end

M.onRaceStart = onRaceStart
M.onRaceWaypointReached = onRaceWaypointReached
M.onRaceTick = onRaceTick

return M
